@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,17 +16,17 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <string.h>
 #include <ctype.h>
+#include <math.h>
 
 #include "sndfile.h"
 #include "sfendian.h"
-#include "float_cast.h"
 #include "common.h"
 
 /*------------------------------------------------------------------------------
@@ -85,10 +85,10 @@ static sf_count_t paf24_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t paf24_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t paf24_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t paf24_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t paf24_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t paf24_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t paf24_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t paf24_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t paf24_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t paf24_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static sf_count_t paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset) ;
 
@@ -113,13 +113,13 @@ paf_open	(SF_PRIVATE *psf)
 			return error ;
 		} ;
 
-	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
+	subformat = SF_CODEC (psf->sf.format) ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_PAF)
+	{	if ((SF_CONTAINER (psf->sf.format)) != SF_FORMAT_PAF)
 			return	SFE_BAD_OPEN_FORMAT ;
 
-		endian = psf->sf.format & SF_FORMAT_ENDMASK ;
+		endian = SF_ENDIAN (psf->sf.format) ;
 
 		/* PAF is by default big endian. */
 		psf->endian = SF_ENDIAN_BIG ;
@@ -163,6 +163,7 @@ paf_read_header	(SF_PRIVATE *psf)
 {	PAF_FMT		paf_fmt ;
 	int			marker ;
 
+	memset (&paf_fmt, 0, sizeof (paf_fmt)) ;
 	psf_binheader_readf (psf, "pm", 0, &marker) ;
 
 	psf_log_printf (psf, "Signature   : '%M'\n", marker) ;
@@ -272,7 +273,7 @@ paf_read_header	(SF_PRIVATE *psf)
 } /* paf_read_header */
 
 static int
-paf_write_header (SF_PRIVATE *psf, int calc_length)
+paf_write_header (SF_PRIVATE *psf, int UNUSED (calc_length))
 {	int			paf_format ;
 
 	/* PAF header already written so no need to re-write. */
@@ -281,12 +282,7 @@ paf_write_header (SF_PRIVATE *psf, int calc_length)
 
 	psf->dataoffset = PAF_HEADER_LENGTH ;
 
-	psf->dataoffset = PAF_HEADER_LENGTH ;
-
-	/* Prevent compiler warning. */
-	calc_length = calc_length ;
-
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 					paf_format = PAF_PCM_S8 ;
 					break ;
@@ -349,7 +345,7 @@ static int paf24_close (SF_PRIVATE *psf) ;
 static int
 paf24_init (SF_PRIVATE *psf)
 {	PAF24_PRIVATE	*ppaf24 ;
-	int	paf24size, max_blocks ;
+	int	paf24size ;
 
 	paf24size = sizeof (PAF24_PRIVATE) + psf->sf.channels *
 					(PAF24_BLOCK_SIZE + PAF24_SAMPLES_PER_BLOCK * sizeof (int)) ;
@@ -360,10 +356,10 @@ paf24_init (SF_PRIVATE *psf)
 	*/
 	psf->last_op = 0 ;
 
-	if (! (psf->fdata = malloc (paf24size)))
+	if (! (psf->codec_data = malloc (paf24size)))
 		return SFE_MALLOC_FAILED ;
 
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 	memset (ppaf24, 0, paf24size) ;
 
 	ppaf24->channels	= psf->sf.channels ;
@@ -390,7 +386,7 @@ paf24_init (SF_PRIVATE *psf)
 		} ;
 
 	psf->seek	= paf24_seek ;
-	psf->close	= paf24_close ;
+	psf->container_close	= paf24_close ;
 
 	psf->filelength = psf_get_filelen (psf) ;
 	psf->datalength = psf->filelength - psf->dataoffset ;
@@ -398,18 +394,18 @@ paf24_init (SF_PRIVATE *psf)
 	if (psf->datalength % PAF24_BLOCK_SIZE)
 	{	if (psf->mode == SFM_READ)
 			psf_log_printf (psf, "*** Warning : file seems to be truncated.\n") ;
-		max_blocks = psf->datalength / ppaf24->blocksize + 1 ;
+		ppaf24->max_blocks = psf->datalength / ppaf24->blocksize + 1 ;
 		}
 	else
-		max_blocks = psf->datalength / ppaf24->blocksize ;
+		ppaf24->max_blocks = psf->datalength / ppaf24->blocksize ;
 
 	ppaf24->read_block = 0 ;
 	if (psf->mode == SFM_RDWR)
-		ppaf24->write_block = max_blocks ;
+		ppaf24->write_block = ppaf24->max_blocks ;
 	else
 		ppaf24->write_block = 0 ;
 
-	psf->sf.frames = ppaf24->samplesperblock * max_blocks ;
+	psf->sf.frames = ppaf24->samplesperblock * ppaf24->max_blocks ;
 	ppaf24->sample_count = psf->sf.frames ;
 
 	return 0 ;
@@ -420,12 +416,12 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 {	PAF24_PRIVATE	*ppaf24 ;
 	int				newblock, newsample ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 	{	psf->error = SFE_INTERNAL ;
-		return SF_SEEK_ERROR ;
+		return PSF_SEEK_ERROR ;
 		} ;
 
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	if (mode == SFM_READ && ppaf24->write_count > 0)
 		paf24_write_block (psf, ppaf24) ;
@@ -435,11 +431,6 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 	switch (mode)
 	{	case SFM_READ :
-				if (offset > ppaf24->read_block * ppaf24->samplesperblock + ppaf24->read_count)
-				{	psf->error = SFE_BAD_SEEK ;
-					return SF_SEEK_ERROR ;
-					} ;
-
 				if (psf->last_op == SFM_WRITE && ppaf24->write_count)
 					paf24_write_block (psf, ppaf24) ;
 
@@ -452,7 +443,7 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 		case SFM_WRITE :
 				if (offset > ppaf24->sample_count)
 				{	psf->error = SFE_BAD_SEEK ;
-					return SF_SEEK_ERROR ;
+					return PSF_SEEK_ERROR ;
 					} ;
 
 				if (psf->last_op == SFM_WRITE && ppaf24->write_count)
@@ -466,7 +457,7 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 		default :
 				psf->error = SFE_BAD_SEEK ;
-				return SF_SEEK_ERROR ;
+				return PSF_SEEK_ERROR ;
 		} ;
 
 	return newblock * ppaf24->samplesperblock + newsample ;
@@ -476,10 +467,10 @@ static int
 paf24_close (SF_PRIVATE *psf)
 {	PAF24_PRIVATE *ppaf24 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
 
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
 	{	if (ppaf24->write_count > 0)
@@ -568,9 +559,9 @@ paf24_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int				k, bufferlen, readcount, count ;
 	sf_count_t		total = 0 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	iptr = psf->u.ibuf ;
 	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
@@ -590,9 +581,9 @@ paf24_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 {	PAF24_PRIVATE *ppaf24 ;
 	int				total ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	total = paf24_read (psf, ppaf24, ptr, len) ;
 
@@ -607,9 +598,9 @@ paf24_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	float			normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 / 0x80000000) : (1.0 / 0x100) ;
 
@@ -634,9 +625,9 @@ paf24_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	double 			normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 / 0x80000000) : (1.0 / 0x100) ;
 
@@ -707,7 +698,7 @@ paf24_write_block (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24)
 } /* paf24_write_block */
 
 static int
-paf24_write (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24, int *ptr, int len)
+paf24_write (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24, const int *ptr, int len)
 {	int		count, total = 0 ;
 
 	while (total < len)
@@ -728,15 +719,15 @@ paf24_write (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24, int *ptr, int len)
 } /* paf24_write */
 
 static sf_count_t
-paf24_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
+paf24_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	PAF24_PRIVATE 	*ppaf24 ;
 	int				*iptr ;
 	int				k, bufferlen, writecount = 0, count ;
 	sf_count_t		total = 0 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	iptr = psf->u.ibuf ;
 	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
@@ -754,14 +745,14 @@ paf24_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 } /* paf24_write_s */
 
 static sf_count_t
-paf24_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
+paf24_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	PAF24_PRIVATE 	*ppaf24 ;
 	int				writecount, count ;
 	sf_count_t		total = 0 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	writecount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -778,16 +769,16 @@ paf24_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 } /* paf24_write_i */
 
 static sf_count_t
-paf24_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
+paf24_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	PAF24_PRIVATE 	*ppaf24 ;
 	int				*iptr ;
 	int				k, bufferlen, writecount = 0, count ;
 	sf_count_t		total = 0 ;
 	float			normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : (1.0 / 0x100) ;
 
@@ -808,16 +799,16 @@ paf24_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 } /* paf24_write_f */
 
 static sf_count_t
-paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
+paf24_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	PAF24_PRIVATE 	*ppaf24 ;
 	int				*iptr ;
 	int				k, bufferlen, writecount = 0, count ;
 	sf_count_t		total = 0 ;
 	double			normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
+	ppaf24 = (PAF24_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : (1.0 / 0x100) ;
 
@@ -837,11 +828,3 @@ paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	return total ;
 } /* paf24_write_d */
 
-
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 477a5308-451e-4bbd-bab4-fab6caa4e884
-*/
